program npng;

uses NLib, NStdIO, ZlibDll;

type
  TIHDR = packed record
    Width: Integer;
    Height: Integer;
    BitDepth: Byte;
    ColorType: Byte;
    Compression: Byte;
    Filter: Byte;
    Interlace: Byte;
  end;

var
  CrcTable: array[0..255] of Cardinal;

function BSwap32(x: Integer): Integer;
begin
  Result := ((x shl 24) and $ff000000) or
      ((x shl 8) and $00ff0000) or
      ((x shr 8) and $0000ff00) or
      ((x shr 24) and $000000ff);
end;

procedure InitCrcTable;
var
  n, k: Integer;
  c: Cardinal;
begin
  for n := 0 to 255 do
  begin
    c := n;
    for k := 0 to 7 do
      if (c and 1) = 1 then
        c := $edb88320 xor (c shr 1)
      else
        c := c shr 1; 
    CrcTable[n] := c;
  end;
end;

function CalcCrc(const ChunkType: string; const Data; Len: Cardinal): Cardinal;
var
  n: Integer;
begin
  Result := $ffffffff;
  for n := 1 to 4 do
    Result := CrcTable[(Result xor Byte(ChunkType[n])) and $ff]
        xor (Result shr 8);
  for n := 0 to Len - 1 do
    Result := CrcTable[(Result xor Byte(PChar(@Data)[n])) and $ff]
        xor (Result shr 8);
  Result := Result xor $ffffffff;
end;

function Compress(InBuf: PChar; InBufSize: Cardinal;
  var OutBuf: PChar; var OutBufSize: Cardinal): Integer;
const
  bsize = 1024;
  //bsize = 32768;
var
  bufsize: Integer;
  z: z_stream; 
  p: PChar;
begin
  FillChar(z, SizeOf(z_stream), 0);
  //deflateInit(z, 6); // TODO: check error
  //deflateInit2(z, Z_DEFAULT_COMPRESSION, Z_DEFLATED, 15, 8, Z_RLE);  
  deflateInit2(z, Z_BEST_SPEED, Z_DEFLATED, 15, 8, Z_FILTERED);  

  bufsize := bsize;
  
  GetMem(OutBuf, bufsize);
  p := OutBuf;
  z.next_in := InBuf;
  z.avail_in := InBufSize;
  z.next_out := p;
  z.avail_out := bufsize; 
  while True do
  begin
    if deflate(z, Z_FINISH) = Z_STREAM_END then // TODO: check error
      Break;
    if z.avail_out = 0 then
    begin
      ReallocMem(p, bufsize + bsize); 
      z.next_out := @p[bufsize];
      z.avail_out := bsize;
      Inc(bufsize, bsize);
    end;
  end;
  OutBuf := p;
  OutBufSize := bufsize - z.avail_out; 
end;

procedure WriteChunk(const F: TFile; const ChunkType: string; const Data;
    Len: Cardinal);
var
  tmp: Cardinal;
begin
  tmp := BSwap32(Len);
  F.Write(tmp, SizeOf(tmp));
  F.Write(ChunkType[1], 4);
  if Len > 0 then
    F.Write(Data, Len);
  tmp := BSwap32(CalcCrc(ChunkType, Data, Len));
  F.Write(tmp, SizeOf(tmp));
end;

function CalcPitch(Width: Integer; Bpp: Integer): Integer;
begin
  Result := ((Width * Bpp + 7) shr 3 + 3) and not 3;
end;

procedure WritePng(const F: TFile; Width, Height, Component: Integer;
  Bits: PChar);
const
  FileSignature = #$89'PNG'#$0D#$0A#$1A#$0A;
var
  ihdr: TIHDR;
  dat: PChar;
  datsize: Cardinal;
  dummy: Integer;
  src, p: PChar;
  h, w: Integer;
  r, g, b: Char;
begin
  F.Write(FileSignature, Length(FileSignature));

  ihdr.Width := BSwap32(Width);
  ihdr.Height := BSwap32(Height);
  ihdr.BitDepth := 8;
  if Component = 1 then
    ihdr.ColorType := 0
  else
    ihdr.ColorType := 2;
  ihdr.Compression := 0;
  ihdr.Filter := 0;
  ihdr.Interlace := 0;
  WriteChunk(F, 'IHDR', ihdr, SizeOf(ihdr));

  GetMem(src, (Width + 1) * Height * Component);
  p := src;
  Bits := @Bits[CalcPitch(Width, 24)*(Height-1)];
  for h := 0 to Height - 1 do
  begin
    p[0] := #1; // sub
    for w := 0 to Width - 1 do
    begin
      r := Bits[0+w*Component+2];
      g := Bits[0+w*Component+1];
      b := Bits[0+w*Component];
      if w = 0 then
      begin
        p[1+w*Component] := r;
        p[1+w*Component+1] := g; 
        p[1+w*Component+2] := b;
      end
      else begin
        p[1+w*Component] := Char(Byte(r) - Byte(Bits[(w-1)*Component+2]));
        p[1+w*Component+1] := Char(Byte(g) - Byte(Bits[(w-1)*Component+1]));
        p[1+w*Component+2] := Char(Byte(b) - Byte(Bits[(w-1)*Component]));
      end;
    end;
    Dec(Bits, CalcPitch(Width, 24));
    Inc(p, Width * Component + 1);
  end;
  Compress(src, (Width * Component + 1) * Height, dat, datsize);
  WriteChunk(F, 'IDAT', dat^, datsize);

  WriteChunk(F, 'IEND', dummy, 0)
end;

var
  f: TFile;
  buf: PChar;
begin
  InitCrcTable;

  try
    f := TFile.Create('peppers-333x333.bmp');
    //f := TFile.Create('a.bmp');
    GetMem(buf, f.Size);
    f.Read(buf^, f.Size);
    f.Free;
  except
    on E: EIOError do SysFatal(E);
  end;
  
  WritePng(StdOut, 333, 333, 3, @buf[36+18]);
  //WritePng(StdOut, 768, 512, 3, @buf[36]);
  //WritePng(StdOut, 8, 8, 1, @buf[36]);
end.
