unit ZLibDll;

interface

const
  ZLIB_VERSION = '1.2.3';

type
  alloc_func = function(opaque: Pointer; items, size: Integer): Pointer;
                 cdecl;
  free_func  = procedure(opaque, address: Pointer);
                 cdecl;

  in_func    = function(opaque: Pointer; var buf: PByte): Integer;
                 cdecl;
  out_func   = function(opaque: Pointer; buf: PByte; size: Integer): Integer;
                 cdecl;

  z_streamp = ^z_stream;
  z_stream = packed record
    next_in: PChar;       (* next input byte *)
    avail_in: Integer;    (* number of bytes available at next_in *)
    total_in: LongInt;    (* total nb of input bytes read so far *)

    next_out: PChar;      (* next output byte should be put there *)
    avail_out: Integer;   (* remaining free space at next_out *)
    total_out: LongInt;   (* total nb of bytes output so far *)

    msg: PChar;           (* last error message, NULL if no error *)
    state: Pointer;       (* not visible by applications *)

    zalloc: alloc_func;   (* used to allocate the internal state *)
    zfree: free_func;     (* used to free the internal state *)
    opaque: Pointer;      (* private data object passed to zalloc and zfree *)

    data_type: Integer;   (* best guess about the data type: ascii or binary *)
    adler: LongInt;       (* adler32 value of the uncompressed data *)
    reserved: LongInt;    (* reserved for future use *)
  end;

(* constants *)
const
  Z_NO_FLUSH      = 0;
  Z_PARTIAL_FLUSH = 1;
  Z_SYNC_FLUSH    = 2;
  Z_FULL_FLUSH    = 3;
  Z_FINISH        = 4;

  Z_OK            =  0;
  Z_STREAM_END    =  1;
  Z_NEED_DICT     =  2;
  Z_ERRNO         = -1;
  Z_STREAM_ERROR  = -2;
  Z_DATA_ERROR    = -3;
  Z_MEM_ERROR     = -4;
  Z_BUF_ERROR     = -5;
  Z_VERSION_ERROR = -6;

  Z_NO_COMPRESSION       =  0;
  Z_BEST_SPEED           =  1;
  Z_BEST_COMPRESSION     =  9;
  Z_DEFAULT_COMPRESSION  = -1;

  Z_FILTERED            = 1;
  Z_HUFFMAN_ONLY        = 2;
  Z_RLE                 = 3;
  Z_DEFAULT_STRATEGY    = 0;

  Z_BINARY   = 0;
  Z_ASCII    = 1;
  Z_UNKNOWN  = 2;

  Z_DEFLATED = 8;

var
  (* basic functions *)
  zlibVersion: function: PChar; cdecl;
  deflate: function (var strm: z_stream; flush: Integer): Integer; cdecl;
  deflateEnd: function (var strm: z_stream): Integer; cdecl;
  inflate: function (var strm: z_stream; flush: Integer): Integer; cdecl;
  inflateEnd: function (var strm: z_stream): Integer; cdecl;

  (* various hacks, don't look :) *)
  deflateInit_: function (var strm: z_stream; level: Integer;
      const version: PChar; stream_size: Integer): Integer; cdecl;
  inflateInit_: function (var strm: z_stream; const version: PChar;
      stream_size: Integer): Integer; cdecl; 
  deflateInit2_: function (var strm: z_stream;
      level, method, windowBits, memLevel, strategy: Integer;
      const version: PChar; stream_size: Integer): Integer; cdecl;

function deflateInit(var strm: z_stream; level: Integer): Integer;
function deflateInit2(var strm: z_stream; level, method, windowBits, memLevel,
                      strategy: Integer): Integer;

implementation

uses Windows;

function deflateInit(var strm: z_stream; level: Integer): Integer;
begin
  Result := deflateInit_(strm, level, ZLIB_VERSION, sizeof(z_stream));
end;

function deflateInit2(var strm: z_stream; level, method, windowBits, memLevel,
                      strategy: Integer): Integer;
begin
  Result := deflateInit2_(strm, level, method, windowBits, memLevel, strategy,
                          ZLIB_VERSION, sizeof(z_stream));
end;

function inflateInit(var strm: z_stream): Integer;
begin
  Result := inflateInit_(strm, ZLIB_VERSION, sizeof(z_stream));
end;

var
  Handle: THandle;
begin
  Handle := LoadLibrary('zlib1.dll');
  if Handle <> 0 then
  begin
    zlibVersion := GetProcAddress(Handle, 'zlibVersion');
    deflateInit_ := GetProcAddress(Handle, 'deflateInit_');
    deflateInit2_ := GetProcAddress(Handle, 'deflateInit2_');
    deflate := GetProcAddress(Handle, 'deflate');
    deflateEnd := GetProcAddress(Handle, 'deflateEnd');
    inflateInit_ := GetProcAddress(Handle, 'inflateInit_');
    inflate := GetProcAddress(Handle, 'inflate');
    inflateEnd := GetProcAddress(Handle, 'inflateEnd');
  end;
end.
